#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <linux/input.h>
#include <linux/fb.h>

#include <android/log.h>
#define LOG_TAG "show infomation"
#define LOGW(a )  __android_log_write(ANDROID_LOG_WARN,LOG_TAG,a)

#define WACOM_DEV "/dev/input/event6"
#define TABLET_DEV "/dev/input/event4" 
#define TABLET_DEV_EE "/dev/input/event3" 

static int ts_fd = -1;
static struct input_event data;

typedef struct 
{
  int wacomStartX;
  int wacomStartY;
  int wacomEndX;
  int wacomEndY;

  int tabletStartX;
  int tabletStartY;
  int tabletEndX;
  int tabletEndY;
}CALIBRATION;

CALIBRATION calibration;

static struct fb_var_screeninfo vi;


int isEE = 0;

static int deviceIsEE()
{

    int fd;
    void *bits;

    fd = open("/dev/graphics/fb0", O_RDWR);
    if(fd < 0) {
        LOGW("cannot open fb0");
        return -1;
    }

    if(ioctl(fd, FBIOGET_VSCREENINFO, &vi) < 0) {
        LOGW("failed to get fb0 info");
        return -1;
    }
    close(fd);
    if(vi.xres == 800)
    {
      LOGW("device is PE.");
      return 0;
    }
    else
    {
      LOGW("device is EE.");
      return 1;
    }
}



static int init_device(char *TS_DEV)
{
    if((ts_fd = open(TS_DEV, O_RDONLY)) < 0)
    {
        LOGW("Error open ");
        LOGW(TS_DEV);
        LOGW("\n");
        return -1;
    }
    return ts_fd;
}


static int calibrateWacomRightTop()
{
    int i = 0;
    int type,code,value;


    if(init_device(WACOM_DEV) < 0)
        return -1;
    while(1)
    {
        read(ts_fd, &data, sizeof(data));
        type = data.type;
        code = data.code;
        value = data.value;


        if(data.type == 3 && data.code == 0) /*abs,x */
        {
          if(calibration.wacomStartX == 0)
            calibration.wacomStartX = data.value;
        }
        else if(data.type == 3 && data.code == 1) /*abs,y*/
        {
          if(calibration.wacomStartY == 0)
            calibration.wacomStartY = data.value;
        }
        if((calibration.wacomStartX != 0)&&(calibration.wacomStartY != 0))
        {
          close(ts_fd);
          return 1;
        }
    }


}

static int calibrateWacomLeftBottom()
{
    int i = 0;
    int type,code,value;


    if(init_device(WACOM_DEV) < 0)
        return -1;
    while(1)
    {
        read(ts_fd, &data, sizeof(data));
        type = data.type;
        code = data.code;
        value = data.value;


        if(data.type == 3 && data.code == 0) /*abs,x */
        {
          if(calibration.wacomEndX == 0)
            calibration.wacomEndX = data.value;
        }
        else if(data.type == 3 && data.code == 1) /*abs,y*/
        {
          if(calibration.wacomEndY == 0)
            calibration.wacomEndY = data.value;
        }
        if((calibration.wacomEndX != 0)&&(calibration.wacomEndY != 0))
        {
          close(ts_fd);
          return 1;
        }
    }


}

static int calibrateTabletLeftTop()
{
    int i = 0;
    int type,code,value;

    if(isEE)
    {
      if(init_device(TABLET_DEV_EE) < 0)
          return -1;
    }
    else 
    {
      if(init_device(TABLET_DEV) < 0)
          return -1;
    }

    while(1)
    {
        read(ts_fd, &data, sizeof(data));
        type = data.type;
        code = data.code;
        value = data.value;


        if(data.type == 3 && data.code == 0) /*abs,x */
        {
          if(calibration.tabletStartX == 0)
            calibration.tabletStartX = data.value;
        }
        else if(data.type == 3 && data.code == 1) /*abs,y*/
        {
          if(calibration.tabletStartY == 0)
            calibration.tabletStartY = data.value;
        }
        if((calibration.tabletStartX != 0)&&(calibration.tabletStartY != 0))
        {
          close(ts_fd);
          return 1;
        }
    }
}

static int calibrateTabletRightBottom()
{
    int i = 0;
    int type,code,value;

    if(isEE)
    {
      if(init_device(TABLET_DEV_EE) < 0)
          return -1;
    }
    else 
    {
      if(init_device(TABLET_DEV) < 0)
          return -1;
    }

    while(1)
    {
        read(ts_fd, &data, sizeof(data));
        type = data.type;
        code = data.code;
        value = data.value;


        if(data.type == 3 && data.code == 0) /*abs,x */
        {
          if(calibration.tabletEndX == 0)
            calibration.tabletEndX = data.value;
        }
        else if(data.type == 3 && data.code == 1) /*abs,y*/
        {
          if(calibration.tabletEndY == 0)
            calibration.tabletEndY = data.value;
        }
        if((calibration.tabletEndX != 0)&&(calibration.tabletEndY != 0))
        {
          close(ts_fd);
          return 1;
        }
    }
}


int main()
{
   FILE  *AStream;
   memset(&calibration,0,sizeof(CALIBRATION));
   isEE= deviceIsEE();
   system("am start -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.CalibrationActivity --ei Step 1 --activity-clear-top");
   calibrateTabletLeftTop();
   system("am start -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.CalibrationActivity --ei Step 2 --activity-clear-top");
   calibrateWacomRightTop();
   system("am start -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.CalibrationActivity --ei Step 3 --activity-clear-top");
   calibrateTabletRightBottom();
   system("am start -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.CalibrationActivity --ei Step 4 --activity-clear-top");
   calibrateWacomLeftBottom();
   system("am start -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.CalibrationActivity --ei Step 5 --activity-clear-top");

   fprintf(stderr,"wacomStartX = %d\n", calibration.wacomStartX);
   fprintf(stderr,"wacomStartY = %d\n", calibration.wacomStartY);
   fprintf(stderr,"wacomEndX = %d\n", calibration.wacomEndX);
   fprintf(stderr,"wacomEndY = %d\n", calibration.wacomEndY);
   fprintf(stderr,"tabletStartX = %d\n", calibration.tabletStartX);
   fprintf(stderr,"tabletStartY = %d\n", calibration.tabletStartY);
   fprintf(stderr,"tabletEndX = %d\n", calibration.tabletEndX);
   fprintf(stderr,"tabletEndY = %d\n", calibration.tabletEndY);

   AStream=fopen("/sdcard/public/calibration.dat","wb");
   if(AStream==NULL) {
      AStream=fopen("/intsdcard/public/calibration.dat","wb");
   }

    if(AStream != NULL)
    {
      fwrite(&calibration,sizeof(CALIBRATION),1,AStream);
      fclose(AStream);
    }

   LOGW("calibration exit.\n");
   return 0;
}