#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <linux/input.h>

#include <android/log.h>
#define LOG_TAG "show infomation"
#define LOGW(a )  __android_log_write(ANDROID_LOG_WARN,LOG_TAG,a)

#define EVKKEY_DEV "/dev/input/event1"
#define TABLET_DEV "/dev/input/event4" 
#define BACKLIGHT_DEV "/sys/class/backlight/pwm-backlight.0/brightness"

int tabletStartX = 170;
int tabletStartY = 84;
int tabletEndX = 3830;
int tabletEndY = 4007;

typedef struct 
{
  int wacomStartX;
  int wacomStartY;
  int wacomEndX;
  int wacomEndY;

  int tabletStartX;
  int tabletStartY;
  int tabletEndX;
  int tabletEndY;
}CALIBRATION;


static int ts_fd = -1;
static struct input_event data;
int sendevent(char* device,int type, int code,int value)
{
    int i;
    int fd;
    int ret;
    struct input_event event;
    char eventmsg[10];
    struct timeval tv;

    tv.tv_sec = 0;
    tv.tv_usec = 2*1000;
    select (0, NULL, NULL, NULL, &tv);  
//    LOGW("sleep for 2 ms before sending event.");

    fd = open(device, O_RDWR);
    if(fd < 0) {
        LOGW("Error open ");
        LOGW(device);
        LOGW("\n");
        return -1;
    }
    memset(&event, 0, sizeof(event));
    event.type = type;
    event.code = code;
    event.value = value;
    gettimeofday(&(event.time), NULL);
    ret = write(fd, &event, sizeof(event));
    close(fd);
    if(ret < sizeof(event)) {
        LOGW("write event failed");
        LOGW("\n");
        close(fd);
        return -1;
    }

    return 0;
}

int switchLcdBacklight()
{
    int i;
    int fd;
    int ret;
    static int savedBrightness;
    char brightness[10];
    int currentBrightness;

    memset(brightness,0,10);


    fd = open(BACKLIGHT_DEV, O_RDWR);
    if(fd < 0) {
        LOGW("Error open ");
        LOGW(BACKLIGHT_DEV);
        LOGW("\n");
        return -1;
    }
    read(fd,brightness,9);
    currentBrightness = atoi(brightness);
    if(currentBrightness == 0)
    {
      sprintf(brightness,"%d",savedBrightness);
      ret = write(fd, brightness, strlen(brightness));
    }
    else
    {
      savedBrightness = currentBrightness;
      sprintf(brightness,"%d",0);
      ret = write(fd, brightness, strlen(brightness));

    }
    
    close(fd);

    return 0;
}



int init_device(char *TS_DEV)
{
    if((ts_fd = open(TS_DEV, O_RDONLY)) < 0)
    {
        LOGW("Error open ");
        LOGW(TS_DEV);
        LOGW("\n");
        return -1;
    }
    return ts_fd;
}

int usePageKeys()
{
    FILE  *AStream;
    AStream=fopen("/sdcard/public/usepagekeys","rb");
    if(AStream == NULL)
    {
       AStream=fopen("/intsdcard/public/usepagekeys.dat","rb");
    }
    if(AStream != NULL)
    {
      fclose(AStream);
      return 1;
    }
    else
    {
      return 0;
    }
}

 int listento_evkkey()
{
    int i = 0;
    struct timeval tvLastAppSwitcherPress;
    struct timeval tvNow;
    if(init_device(EVKKEY_DEV) < 0)
        return -1;
    static int y = 0;
    int step = (tabletEndX-tabletStartX)/40;
    while(1)
    {
        read(ts_fd, &data, sizeof(data));
        if (data.type == EV_KEY && data.code == 183 && data.value == 1) /*it's an app switcher key press*/
        {
          gettimeofday(&tvLastAppSwitcherPress,NULL);
        }
        else if(data.type == EV_KEY && data.code == 183 && data.value == 0)/*it's an app switcher key release*/
        {
          gettimeofday(&tvNow,NULL);
          if(tvNow.tv_sec-tvLastAppSwitcherPress.tv_sec >=1) /* press the key for more than 1 seconds will switch the backlight of lcd*/
          {
            switchLcdBacklight();
          }
          else
          {
            system("am startservice -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.LCD2EinkService --ei Mode 0");
          }
        }
        else if(data.type == EV_KEY && data.code == 183 && data.value == 100)
        {
        /*    system("sendevent /dev/input/event1 1 183 0");*/
            close(ts_fd);            
            return 0;
        }
        
        else if(data.type == EV_KEY && data.code == 109 && data.value == 1 && usePageKeys() == 1)
        {
            LOGW("next page key pressed.\n");
            
            system("echo 0 >>/sys/class/graphics/fb2/do_display");
            
            sendevent(TABLET_DEV,EV_ABS,ABS_Y,(tabletEndY-tabletStartY)/2);
            sendevent(TABLET_DEV,EV_ABS,ABS_X,tabletEndX-50);
            sendevent(TABLET_DEV,EV_ABS,ABS_PRESSURE,255);
            sendevent(TABLET_DEV,EV_KEY,BTN_TOUCH,1);
            sendevent(TABLET_DEV,0,0,0);

            y = (y+1)%2;

            
            for(i=1;tabletEndX-50-i*step>tabletStartX+50;i++)
            {
              sendevent(TABLET_DEV,EV_ABS,ABS_Y,(tabletEndY-tabletStartY)/2+y);
              sendevent(TABLET_DEV,EV_ABS,ABS_X,tabletEndX-50-i*step);
              sendevent(TABLET_DEV,0,0,0);
              y = (y+1)%2;
            }
            

            sendevent(TABLET_DEV,EV_ABS,ABS_PRESSURE,0);
            sendevent(TABLET_DEV,EV_KEY,BTN_TOUCH,0);
            sendevent(TABLET_DEV,0,0,0);
            y = (y+1)%2;
            /*

           sendevent(TABLET_DEV,EV_ABS,ABS_Y,tabletEndY-10);
            sendevent(TABLET_DEV,EV_ABS,ABS_X,(tabletEndX-tabletStartX)/2);
            sendevent(TABLET_DEV,EV_ABS,ABS_PRESSURE,255);
            sendevent(TABLET_DEV,EV_KEY,BTN_TOUCH,1);
            sendevent(TABLET_DEV,0,0,0);

            for(i=1;tabletEndY-10-i*50>tabletStartY;i++)
            {
              sendevent(TABLET_DEV,EV_ABS,ABS_Y,tabletEndY-10-i*50);
              sendevent(TABLET_DEV,EV_ABS,ABS_X,(tabletEndX-tabletStartX)/2);
              sendevent(TABLET_DEV,0,0,0);
            }

            sendevent(TABLET_DEV,EV_ABS,ABS_Y,tabletEndY-10-i*50);
            sendevent(TABLET_DEV,EV_ABS,ABS_X,(tabletEndX-tabletStartX)/2);
            sendevent(TABLET_DEV,EV_ABS,ABS_PRESSURE,0);
            sendevent(TABLET_DEV,EV_KEY,BTN_TOUCH,0);
            sendevent(TABLET_DEV,0,0,0);
            */

            system("am startservice -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.LCD2EinkService --ei Mode 3");

        }
        else if(data.type == EV_KEY && data.code == 104 && data.value == 1 && usePageKeys() == 1)
        {
            LOGW("previous page key pressed.\n");

            system("echo 0 >>/sys/class/graphics/fb2/do_display");

            sendevent(TABLET_DEV,EV_ABS,ABS_Y,(tabletEndY-tabletStartY)/2);
            sendevent(TABLET_DEV,EV_ABS,ABS_X,tabletStartX+50);
            sendevent(TABLET_DEV,EV_ABS,ABS_PRESSURE,255);
            sendevent(TABLET_DEV,EV_KEY,BTN_TOUCH,1);
            sendevent(TABLET_DEV,0,0,0);

            y = (y+1)%2;
            

            for(i=1;tabletStartX+50+i*step<tabletEndX-50;i++)
            {
              sendevent(TABLET_DEV,EV_ABS,ABS_Y,(tabletEndY-tabletStartY)/2);
              sendevent(TABLET_DEV,EV_ABS,ABS_X,tabletStartX+50+i*step);
              sendevent(TABLET_DEV,0,0,0);
              y = (y+1)%2;
            }


            sendevent(TABLET_DEV,EV_ABS,ABS_PRESSURE,0);
            sendevent(TABLET_DEV,EV_KEY,BTN_TOUCH,0);
            sendevent(TABLET_DEV,0,0,0);

            y = (y+1)%2;

            /*

            sendevent(TABLET_DEV,EV_ABS,ABS_Y,tabletStartY+10);
            sendevent(TABLET_DEV,EV_ABS,ABS_X,(tabletEndX-tabletStartX)/2);
            sendevent(TABLET_DEV,EV_ABS,ABS_PRESSURE,255);
            sendevent(TABLET_DEV,EV_KEY,BTN_TOUCH,1);
            sendevent(TABLET_DEV,0,0,0);

            for(i=1;tabletStartY+10+i*50<tabletEndY;i++)
            {
              sendevent(TABLET_DEV,EV_ABS,ABS_Y,tabletStartY+10+i*50);
              sendevent(TABLET_DEV,EV_ABS,ABS_X,(tabletEndX-tabletStartX)/2);
              sendevent(TABLET_DEV,0,0,0);
            }

            sendevent(TABLET_DEV,EV_ABS,ABS_Y,tabletStartY+10+i*50);
            sendevent(TABLET_DEV,EV_ABS,ABS_X,(tabletEndX-tabletStartX)/2);
            sendevent(TABLET_DEV,EV_ABS,ABS_PRESSURE,0);
            sendevent(TABLET_DEV,EV_KEY,BTN_TOUCH,0);
            sendevent(TABLET_DEV,0,0,0);
            
*/
            system("am startservice -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.LCD2EinkService --ei Mode 3");

        }      
        
    }
    close(ts_fd);
    return 0;
}

int main()
{
    CALIBRATION calibration;
    FILE  *AStream;
    AStream=fopen("/sdcard/public/calibration.dat","rb");
    if(AStream == NULL)
    {
       AStream=fopen("/intsdcard/public/calibration.dat","rb");
    }

    if(AStream != NULL)
    {
      fread(&calibration,sizeof(CALIBRATION),1,AStream);

      tabletStartX = calibration.tabletStartX;
      tabletStartY = calibration.tabletStartY;
      tabletEndX = calibration.tabletEndX;
      tabletEndY = calibration.tabletEndY;

      fclose(AStream);
    }
   listento_evkkey();
   LOGW("evklistener exit.\n");
   return 0;
}