#include <jni.h>

#include <stdlib.h>
#include <unistd.h>

#include <fcntl.h>
#include <stdio.h>

#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <time.h>
#include <sys/time.h>;

#include <linux/fb.h>
#include <linux/kd.h>

#include "pixelflinger.h"

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include <android/log.h>
#define LOG_TAG "show infomation"
#define LOGW(a )  __android_log_write(ANDROID_LOG_WARN,LOG_TAG,a)

typedef int                 BOOL;
typedef unsigned char       BYTE;
typedef unsigned short      WORD;
typedef unsigned long       DWORD;

#define FALSE               0
#define TRUE                1

BOOL  GetBMPSize(FILE *BMPStream,long *offset);
void  Write_CrossReferenceTable(FILE *AStream,DWORD ObjectPosArray[],int Count);
void  Write_ContentsObject(FILE *AStream,DWORD ObjectPosArray[],int *ObjectIndex,int w,int h);
BOOL BMPtoPDF(BYTE * BitsData,long width,long height,short bits);
BOOL SaveAsBMP(BYTE * BitsData,long width,long height,short bits);


typedef struct {
     long filesize;
     char reserved[2];
     long headersize;
     long infoSize;
     long width;
     long depth;
     short biPlanes;
     short bits;
     long biCompression;
     long biSizeImage;
     long biXPelsPerMeter;
     long biYPelsPerMeter;
     long biClrUsed;
     long biClrImportant;
} BMPHEAD;

//copyright text
char cprght[255]="Copyright(C) 2006-2009 Teksoft SRL, (C)2009 Motisan Radu PFA, All rights reserved.\nwww.teksoftco.com";
//surface pointer
static GGLSurface gr_framebuffer[2];
//handler
static int gr_fb_fd = -1;
//v screen info
static struct fb_var_screeninfo vi;
//f screen info
struct fb_fix_screeninfo fi;

static void dumpinfo(struct fb_fix_screeninfo *fi,
                     struct fb_var_screeninfo *vi);

static int get_framebuffer(GGLSurface *fb)
{
    int fd;
    void *bits;

    fd = open("/dev/graphics/fb0", O_RDWR);
    if(fd < 0) {
        LOGW("cannot open fb0");
        return -1;
    }

    if(ioctl(fd, FBIOGET_FSCREENINFO, &fi) < 0) {
        LOGW("failed to get fb0 info");
        return -1;
    }

    if(ioctl(fd, FBIOGET_VSCREENINFO, &vi) < 0) {
        LOGW("failed to get fb0 info");
        return -1;
    }

    //dumpinfo(&fi, &vi);

    bits = mmap(0, fi.smem_len, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
    if(bits == MAP_FAILED) {
        LOGW("failed to mmap framebuffer");
        return -1;
    }
	
    fb->version = sizeof(*fb);
    fb->width = vi.xres;
    fb->height = vi.yres;
    fb->stride = fi.line_length / (vi.bits_per_pixel >> 3);
    fb->data = bits;
    fb->format = GGL_PIXEL_FORMAT_RGB_565;

    fb++;

    fb->version = sizeof(*fb);
    fb->width = vi.xres;
    fb->height = vi.yres;
    fb->stride = fi.line_length / (vi.bits_per_pixel >> 3);
    fb->data = (void*) (((unsigned) bits) + vi.yres * vi.xres * 2);
    fb->format = GGL_PIXEL_FORMAT_RGB_565;

    return fd;
}


static void dumpinfo(struct fb_fix_screeninfo *fi, struct fb_var_screeninfo *vi)
{
    fprintf(stderr,"vi.xres = %d\n", vi->xres);
    fprintf(stderr,"vi.yres = %d\n", vi->yres);
    fprintf(stderr,"vi.xresv = %d\n", vi->xres_virtual);
    fprintf(stderr,"vi.yresv = %d\n", vi->yres_virtual);
    fprintf(stderr,"vi.xoff = %d\n", vi->xoffset);
    fprintf(stderr,"vi.yoff = %d\n", vi->yoffset);
    fprintf(stderr, "vi.bits_per_pixel = %d\n", vi->bits_per_pixel);

    fprintf(stderr, "fi.line_length = %d\n", fi->line_length);

}


void screen2pdf() 
{
  //get screen capture
  gr_fb_fd = get_framebuffer(gr_framebuffer);
  if (gr_fb_fd <= 0) return;

  int w = vi.xres, h = vi.yres, depth = vi.bits_per_pixel;
  	
  //convert pixel data
  uint8_t *rgb24;
	rgb24 = (uint8_t *)malloc(w * h * 3); 
  if (depth == 16)
  {
	  int i = 0;
	  for (;i<w*h;i++)
	  {
		  uint16_t pixel16 = ((uint16_t *)gr_framebuffer[0].data)[i];
		  // RRRRRGGGGGGBBBBBB -> RRRRRRRRGGGGGGGGBBBBBBBB
		  // in rgb24 color max is 2^8 per channel (*255/32 *255/64 *255/32)
		  rgb24[3*i+2]   = (255*(pixel16 & 0x001F))/ 32; 		//Blue
		  rgb24[3*i+1]   = (255*((pixel16 & 0x07E0) >> 5))/64;	//Green
		  rgb24[3*i]     = (255*((pixel16 & 0xF800) >> 11))/32; 	//Red
	  }
  }
  else
  if (depth == 24)
  {
  	memcpy(rgb24,(uint8_t *) gr_framebuffer[0].data,sizeof(rgb24));
  }
  else 
  {
  	//free
    free(rgb24);
    munmap(gr_framebuffer[0].data, fi.smem_len);
    close(gr_fb_fd);
	  return;
  };

  BMPtoPDF(rgb24,w,h,depth);
  SaveAsBMP(rgb24,w,h,depth);
  free(rgb24);
  munmap(gr_framebuffer[0].data, fi.smem_len);
  close(gr_fb_fd);
  return;
}

int GetLineWidth(WORD biBitCount, int biWidth) {
    return ((biBitCount * biWidth + 15) / 16) * 2 ;
}

BOOL TurnBits(BYTE *BitsData,int BitsSize,int biWidth,WORD biBitCount) {
    int i,j,LineWidth=0;
    BYTE *buffer,*Linebuffer,tmp;

    switch (biBitCount) {
    case     1:
        ;
    case     4:
        LineWidth=GetLineWidth(biBitCount,biWidth);
        break;
    case     8:
        LineWidth=biWidth;
        break;
    case    24:
        LineWidth=biWidth*3;
        break;
    }

    Linebuffer=(BYTE *)malloc(LineWidth);
    if (Linebuffer==NULL)
        return FALSE;

    buffer=(BYTE *)malloc(BitsSize);
    if (buffer==NULL) {
        free(Linebuffer);
        return FALSE;
    }

    if (biBitCount==24) {
        for (i=0;i< (BitsSize / LineWidth);i++) {
            memcpy(Linebuffer,BitsData+(LineWidth*i),LineWidth);

            /* BGR<->RGB?Q */
            for (j=0;j<LineWidth ;j+=3) {
                tmp             = Linebuffer[j];
                Linebuffer[j]   = Linebuffer[j+2];
                Linebuffer[j+2] = tmp;
            }
            memcpy(buffer+ (BitsSize- LineWidth * (i+1) ),Linebuffer,LineWidth);
        }
    } else {
        for (i=0;i< (BitsSize / LineWidth);i++) {
            memcpy(Linebuffer,BitsData+(LineWidth*i),LineWidth);
            memcpy(buffer+ (BitsSize- LineWidth * (i+1) ),Linebuffer,LineWidth);
        }
    }

    memcpy(BitsData,buffer,BitsSize);

    free(buffer);
    free(Linebuffer);

    return(TRUE);
}


void Write_CrossReferenceTable(FILE *AStream,DWORD ObjectPosArray[],int Count) {
    int i;

    fprintf(AStream,"xref\n");
    fprintf(AStream,"0 %d\n",Count+1);
    fprintf(AStream,"0000000000 65535 f \n");

    for (i= 0; i<=Count-1;i++)
        fprintf(AStream,"%0.10d 00000 n \n",ObjectPosArray[i]);
}

void Write_ContentsObject(FILE *AStream,DWORD ObjectPosArray[],int *ObjectIndex,int w,int h) {
    int Length;

    ObjectPosArray[*ObjectIndex]  =(DWORD)ftell(AStream);
    fprintf(AStream,"%d 0 obj\n",*ObjectIndex+1);
    fprintf(AStream,"<< /Length %d 0 R >>\n",*ObjectIndex+2);
    fprintf(AStream,"stream\n");

    /* stream */
    Length=ftell(AStream);
    fprintf(AStream,"q\n");
    fprintf(AStream,"%d 0 0 %d 0 0 cm\n",w,h);
    fprintf(AStream,"/Im0 Do\n");
    fprintf(AStream,"Q\n");
    Length=ftell(AStream)-Length;

    fprintf(AStream,"endstream\n");
    fprintf(AStream,"endobj\n");
    *ObjectIndex=*ObjectIndex+1;

    /* stream Length */
    ObjectPosArray[*ObjectIndex] =(DWORD)ftell(AStream);
    fprintf(AStream,"%d 0 obj\n",*ObjectIndex+1);
    fprintf(AStream,"%d\n",Length);
    fprintf(AStream,"endobj\n");
    *ObjectIndex=*ObjectIndex+1;
}

BOOL BMPtoPDF(BYTE * BitsData,long width,long height,short bits) {
    int   ObjectIndex;
    DWORD ObjectPosArray[10];
    FILE  *AStream;

    int i,j=0;
    DWORD BitsSize;         


    ObjectIndex=0;

    if(BitsData == NULL)
      return FALSE;

    AStream=fopen("/sdcard/public/screen.pdf","wb+");
    if(AStream==NULL) {
        AStream=fopen("/intsdcard/public/screen.pdf","wb+");
        if(AStream==NULL) {
          LOGW("Error : Can not Create File.\n");
          return(-1);
        }
    }

    fprintf(AStream,"%%PDF-1.2\n");

    /* Catalog */
    ObjectPosArray[ObjectIndex] =ftell(AStream);
    fprintf(AStream,"%d 0 obj\n",ObjectIndex+1);
    fprintf(AStream,"<<\n");
    fprintf(AStream,"/Type /Catalog\n");
    fprintf(AStream,"/Pages 2 0 R\n");
    /* View Option (100%) */
    fprintf(AStream,"/OpenAction [3 0 R /XYZ -32768 -32768 1 ]\n");
    fprintf(AStream,">>\n");
    fprintf(AStream,"endobj\n");
    ObjectIndex++;

    /* Parent Pages */
    ObjectPosArray[ObjectIndex] =ftell(AStream);
    fprintf(AStream,"%d 0 obj\n",ObjectIndex+1);
    fprintf(AStream,"<<\n");
    fprintf(AStream,"/Type /Pages\n");
    fprintf(AStream,"/Kids [ 3 0 R ]\n");
    fprintf(AStream,"/Count 1\n");
    fprintf(AStream,">>\n");
    fprintf(AStream,"endobj\n");
    ObjectIndex++;

    /* Kids Page */
    ObjectPosArray[ObjectIndex] =ftell(AStream);
    fprintf(AStream,"%d 0 obj\n",ObjectIndex+1);
    fprintf(AStream,"<<\n");
    fprintf(AStream,"/Type /Page\n");
    fprintf(AStream,"/Parent 2 0 R\n");
    fprintf(AStream,"/Resources\n");
    fprintf(AStream,"<<\n");
    fprintf(AStream,"/XObject << /Im0 4 0 R >>\n");

    // ProcSet

    fprintf(AStream,"/ProcSet [ /PDF /ImageC ]\n");

    fprintf(AStream,">>\n");
    fprintf(AStream,"/MediaBox [ 0 0 %d %d ]\n",width,abs(height));


    fprintf(AStream,"/Contents 5 0 R\n");

    fprintf(AStream,">>\n");
    fprintf(AStream,"endobj\n");
    ObjectIndex++;

    /* XObject Resource */
    ObjectPosArray[ObjectIndex] =ftell(AStream);
    fprintf(AStream,"%d 0 obj\n",ObjectIndex+1);
    fprintf(AStream,"<<\n");
    fprintf(AStream,"/Type /XObject\n");
    fprintf(AStream,"/Subtype /Image\n");
    fprintf(AStream,"/Name /Im0\n");
    fprintf(AStream,"/Width %d\n",width);
    fprintf(AStream,"/Height %d\n",height);

    fprintf(AStream,"/Filter []\n");

    /* BitsPerComponent */

    fprintf(AStream,"/BitsPerComponent 8\n");

    /* ColorSpace */

    fprintf(AStream,"/ColorSpace /DeviceRGB\n");

    BitsSize =width*height*3;
    fprintf(AStream,"/Length %d >>\n",BitsSize);

    fprintf(AStream,"stream\n");

 //   TurnBits(BitsData,BitsSize,width,bits);
    fwrite(BitsData,1,BitsSize,AStream);

    fprintf(AStream,"\nendstream\n");
    fprintf(AStream,"endobj\n");
    ObjectIndex++;

    /* Contents Stream & Object */
    Write_ContentsObject(AStream,ObjectPosArray,&ObjectIndex,width,height);

    /* CrossReferenceTable */
    ObjectPosArray[ObjectIndex] =ftell(AStream);
    Write_CrossReferenceTable(AStream,ObjectPosArray,(int)ObjectIndex);

    /* trailer */
    fprintf(AStream,"trailer\n");
    fprintf(AStream,"<<\n");
    fprintf(AStream,"/Size %d\n",ObjectIndex+1);
    fprintf(AStream,"/Root 1 0 R\n");
    fprintf(AStream,">>\n");
    fprintf(AStream,"startxref\n");
    fprintf(AStream,"%d\n",ObjectPosArray[ObjectIndex]);
    fprintf(AStream,"%%%%EOF\n");

    fclose(AStream);

    LOGW("\nSuccess!\n");

    return (TRUE);
}

BOOL SaveAsBMP(BYTE * BitsData,long width,long height,short bits) {
 //save RGB 24 Bitmap
  int bytes_per_pixel = 3;
  BMPHEAD bh;
  memset ((char *)&bh,0,sizeof(BMPHEAD)); // sets everything to 0
  //bh.filesize  =   calculated size of your file (see below)
  //bh.reserved  = two zero bytes
  bh.headersize  = 54L;			// for 24 bit images
  bh.infoSize  =  0x28L;		// for 24 bit images
  bh.width     = height;			// width of image in pixels
  bh.depth     = width;			// height of image in pixels
  bh.biPlanes  =  1;			// for 24 bit images
  bh.bits      = 8 * bytes_per_pixel;	// for 24 bit images
  bh.biCompression = 0L;		// no compression
  int bytesPerLine;
  bytesPerLine = height * bytes_per_pixel;  	// for 24 bit images
  //round up to a dword boundary
  if (bytesPerLine & 0x0003)
  {
    	bytesPerLine |= 0x0003;
    	++bytesPerLine;
  }
  bh.filesize = bh.headersize + (long)bytesPerLine * bh.depth;
  FILE * bmpfile;
  //printf("Bytes per line : %d\n", bytesPerLine);
   bmpfile=fopen("/sdcard/public/screen.bmp","wb");
    if(bmpfile==NULL) {
        bmpfile=fopen("/intsdcard/public/screen.bmp","wb");
        if(bmpfile==NULL) {
          LOGW("Error : Can not Create File.\n");
          return FALSE;
        }
    }

  if (bmpfile == NULL)
  {
  	return FALSE;
  }
  fwrite("BM",1,2,bmpfile);
  fwrite((char *)&bh, 1, sizeof (bh), bmpfile);
  //fwrite(rgb24,1,w*h*3,bmpfile);
  char *linebuf;
  linebuf = (char *) malloc(bytesPerLine);
  if (linebuf == NULL)
  {
     	fclose(bmpfile);
	    return  FALSE;
  }
 

//Now the next thing to do is to align the BGR triplets and to switch order , to match the BMP RGB24 color format.

 
  int line,x;
  for (line = width-1; line >= 0; line --)
  {
    	// fill line linebuf with the image data for that line
	for( x =height-1 ; x >= 0; x-- )
  	{
   		*(linebuf+((height-1)-x)*bytes_per_pixel) = *(BitsData + (x*width+line)*bytes_per_pixel+2);
   		*(linebuf+((height-1)-x)*bytes_per_pixel+1) = *(BitsData + (x*width+line)*bytes_per_pixel+1);
   		*(linebuf+((height-1)-x)*bytes_per_pixel+2) = *(BitsData + (x*width+line)*bytes_per_pixel+0);
  	}
	// remember that the order is BGR and if width is not a multiple
       	// of 4 then the last few bytes may be unused
	fwrite(linebuf, 1, bytesPerLine, bmpfile);
  }
  fclose(bmpfile);

    LOGW("\nsave as bmp Success!\n");

    return TRUE;
}

void Java_com_javese_android_lcd2einkapp_LCD2EinkService_screen2pdf(JNIEnv *env, jobject obj)
{
	 LOGW("Java_com_javese_android_lcd2einkapp_LCD2EinkService_screen2pdf");
//   screen2pdf();
}

jint JNI_OnLoad(JavaVM* vm, void* reserved)
{
    void *venv;
    LOGW("JNI_OnLoad!");

    if ((*vm)->GetEnv(vm, (void**)&venv, JNI_VERSION_1_4) != JNI_OK) {
        LOGW("ERROR: GetEnv failed");
        return -1;
    }

     return JNI_VERSION_1_4;
}

 /*
jstring
Java_com_example_hellojni_HelloJni_stringFromJNI( JNIEnv* env,
                                                  jobject thiz )
{
    return (*env)->NewStringUTF(env, "Hello from JNI !");
}
*/
jstring
Java_com_javese_android_lcd2einkapp_LCD2EinkService_stringFromJNI( JNIEnv* env,
                                                  jobject thiz )
{
	  LOGW("hell,jni");
    return (*env)->NewStringUTF(env, "Hello from JNI !");
}

int main(int argc,char *argv[]) {
/*	if(argc < 2)
		return(1);
    */
  struct timeval tv;

  tv.tv_sec = 0;
  tv.tv_usec = 300*1000;
 // select (0, NULL, NULL, NULL, &tv);  
 // LOGW("sleep for 300 ms.");

  screen2pdf();
  system("echo 1 >>/sys/class/graphics/fb2/do_display");
  return 1;
}
