#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <linux/input.h>
#include <linux/fb.h>

#include <android/log.h>
#define LOG_TAG "show infomation"
#define LOGW(a )  __android_log_write(ANDROID_LOG_WARN,LOG_TAG,a)

#define WACOM_DEV "/dev/input/event6"
#define TABLET_DEV "/dev/input/event4" 
#define TABLET_DEV_EE "/dev/input/event3" 

#define ROTATE_DEV "/sys/class/graphics/fb2/rotate"

static int ts_fd = -1;
static struct input_event data;
int wacomStartX = 0;
int wacomStartY = 70;
int wacomEndX = 780;
int wacomEndY = 512;

int tabletStartX = 170;
int tabletStartY = 84;
int tabletEndX = 3830;
int tabletEndY = 4007;

typedef struct 
{
  int wacomStartX;
  int wacomStartY;
  int wacomEndX;
  int wacomEndY;

  int tabletStartX;
  int tabletStartY;
  int tabletEndX;
  int tabletEndY;
}CALIBRATION;

static struct fb_var_screeninfo vi;


static int deviceIsEE()
{

    int fd;
    void *bits;

    fd = open("/dev/graphics/fb0", O_RDWR);
    if(fd < 0) {
        LOGW("cannot open fb0");
        return -1;
    }

    if(ioctl(fd, FBIOGET_VSCREENINFO, &vi) < 0) {
        LOGW("failed to get fb0 info");
        close(fd);
        return -1;
    }
    close(fd);
    if(vi.xres == 800)
    {
      LOGW("device is PE.");
      return 0;
    }
    else
    {
      LOGW("device is EE.");
      return 1;
    }
}

int sendevent(char* device,int type, int code,int value)
{
    int i;
    int fd;
    int ret;
    struct input_event event;


    fd = open(device, O_RDWR);
    if(fd < 0) {
        LOGW("Error open ");
        LOGW(device);
        LOGW("\n");
        return -1;
    }
    memset(&event, 0, sizeof(event));
    event.type = type;
    event.code = code;
    event.value = value;
    gettimeofday(&(event.time), NULL);
    ret = write(fd, &event, sizeof(event));
    close(fd);
    if(ret < sizeof(event)) {
        LOGW("write event failed");
        LOGW("\n");
        return -1;
    }
    return 0;
}



static int init_device(char *TS_DEV)
{
    if((ts_fd = open(TS_DEV, O_RDONLY)) < 0)
    {
        LOGW("Error open ");
        LOGW(TS_DEV);
        LOGW("\n");
        return -1;
    }
    return ts_fd;
}


static int listento_wacom()
{
    int i = 0;
    int type,code,value;
    int isEE= deviceIsEE();
    int toCapture = 0;


    if(init_device(WACOM_DEV) < 0)
        return -1;
    while(1)
    {
        read(ts_fd, &data, sizeof(data));
        type = data.type;
        code = data.code;
        value = data.value;
        int fd;
        int ret;
        char rotate[10];
        int rotateValue;

        memset(rotate,0,10);


        fd = open(ROTATE_DEV, O_RDWR);
        if(fd < 0) {
            LOGW("Error open ");
            LOGW(ROTATE_DEV);
            LOGW("\n");
            return -1;
        }
        read(fd,rotate,9);
        rotateValue = atoi(rotate);
        close(fd);

        if(data.type == 3 && data.code == 0 && data.value == 99999)
        {
            close(ts_fd);            
            return 0;
        }

        if(data.type == 3 && data.code == 0) /*abs,x should be mapped to abs,y*/
        {
         
          if(rotateValue != 270)
          {
            code = 1;
            value = (data.value-wacomStartX)*(tabletEndY-tabletStartY)/(wacomEndX-wacomStartX)+tabletStartY;
          }
          else
          {
            value = tabletStartX+(value-wacomEndX)*(tabletEndX-tabletStartX)/(wacomStartX-wacomEndX);
          }
          

        /*  value = (value-64)*(tabletEndX-tabletStartX)/(534-64)+tabletStartX;*/
        }
        else if(data.type == 3 && data.code == 1) /*abs,y should be mapped to abs,x*/
        {
          if(rotateValue != 270)
          {
            code = 0;
            value = tabletEndX-(value-wacomStartY)*(tabletEndX-tabletStartX)/(wacomEndY-wacomStartY);
          }
          else
          {
            value = tabletStartY+(value-wacomStartY)*(tabletEndY-tabletStartY)/(wacomEndY-wacomStartY);;
          }
          
      /*    value = value*(tabletEndY-tabletStartY)/784+tabletStartY;*/
        }
        else if(data.type == 3 && data.code == 24) /*abs,pressure level */
        {
          value = value/2;
        }
        else if(data.code == 272) /*wacom abs,272 should be mapped to abs,BTN_TOUCH*/
        {
          code = BTN_TOUCH;
        }
        if(isEE == 0)
        {
          sendevent(TABLET_DEV,type,code,value);
        }
        else
        {
          sendevent(TABLET_DEV_EE,type,code,value);
        }

        if(code == BTN_TOUCH && value == 0)/*pen up will trigger a capture*/
        {
          toCapture = 1;
        }
        else if(type==0 && code==0 && value==0 && toCapture==1 )/*pen up will trigger a capture*/
        {
          toCapture = 0;
          system("am startservice -n com.javese.android.lcd2einkapp/com.javese.android.lcd2einkapp.LCD2EinkService --ei Mode 4");
        }

    }
    close(ts_fd);
    return 0;
}

int main()
{
    CALIBRATION calibration;
    FILE  *AStream;
    AStream=fopen("/sdcard/public/calibration.dat","rb");
    if(AStream == NULL)
    {
       AStream=fopen("/intsdcard/public/calibration.dat","rb");
    }

    if(AStream != NULL)
    {
      fread(&calibration,sizeof(CALIBRATION),1,AStream);
      wacomStartX = calibration.wacomStartX;
      wacomStartY = calibration.wacomStartY;
      wacomEndX = calibration.wacomEndX;
      wacomEndY = calibration.wacomEndY;

      tabletStartX = calibration.tabletStartX;
      tabletStartY = calibration.tabletStartY;
      tabletEndX = calibration.tabletEndX;
      tabletEndY = calibration.tabletEndY;

      fclose(AStream);
    }

   system("chmod 666 /dev/graphics/fb2"); 
   listento_wacom();
   LOGW("wacomlistener exit.\n");
   return 0;
}